/* This program is licensed under GPL v2 or later. See COPYING */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/uio.h>
#include <lzo/lzo1x.h>

char in[65536]; // extra byte + padding reduces cache aliasing
char out[65536];
char signature[4] = "BLZO";


int main(int argc, char ** argv)
{
	ssize_t r, w;
	lzo_uint c;
	int res;
	unsigned char size[2];
	void * lzomem = malloc(LZO1X_1_MEM_COMPRESS);

	if (!lzomem) {
		write(2, "malloc failed", 12);
		exit(1);
	}
	struct iovec writevec_u[2] = {
		size, 2,
		in, 23,
	};
	struct iovec writevec_c[2] = {
		size, 2,
		out, 23,
	};

	res=write(1, signature, 4);
	if (res != 4) {
		if (res < 0)
			perror("write");
		else
			write(2, "short write\n", 12);
		exit(3);
	}
	while (r = read(0, in, 65536)) {
		if (r < 0) {
			perror("read");
			exit(2);
		}
		c = 65536;
		res = lzo1x_1_compress(in, r, out, &c, lzomem);
		if (res == LZO_E_OK && c < r) {
			size[0] = ((unsigned) c) >> 8;
			size[1] = (unsigned) c;
			writevec_c[1].iov_len = c;
			w = writev(1, writevec_c, 2);
			if (w < 2+c) {
				if (w < 0)
					perror("write");
				else
					write(2, "short write\n", 12);
				exit(3);
			}
		} else {
			writevec_u[1].iov_len = r;
			size[0] = size[1] = 0;
			w = writev(1, writevec_u, 2);
			if (w < 2+r) {
				perror("write");
				exit(3);
			}
		}
	}
}
