/* This program is licensed under GPL v2 or later. See COPYING */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <zlib.h>
#include <errno.h>
#include <sys/uio.h>
#include <lzo1x.h>

char in[65536]; // extra byte + padding reduces cache aliasing
char out[65536];
char signature[4];

int main(int argc, char ** argv)
{
	ssize_t c, r;
	lzo_uint w;
	int res;
	unsigned char size[2];
	struct iovec writevec_u[2] = {
		size, 2,
		in, 23,
	};
	struct iovec writevec_c[2] = {
		size, 2,
		out, 23,
	};
	r = read(0, signature, 4);
	if (r != 4) {
		if (r < 0)
			perror("read");
		else
			write(2, "short read\n", 11);
		exit(3);
	}
	if (memcmp(signature, "BLZO", 4)) {
		write(2, "bad signature\n", 14);
		exit(4);
	}
	while (r = read(0, size, 2)) {
		if (r < 0) {
			perror("read");
			exit(2);
		} else if (r<2) {
			write(2, "short read\n", 11);
			exit(2);
		}
		if (size[0]|size[1]) {
			c = (size[0] << 8) + size[1];
			r = read(0, in, c);
			if (r != c) {
				if (r < 0)
					perror("read");
				else
					write(2, "short read\n", 11);
				exit(2);
			}
			w = 65536;
			res = lzo1x_decompress(in, c, out, &w, NULL);
			if (res != LZO_E_OK) {
				write(2, "uncompress error\n", 17);
				exit(3);
			}
			res = write(1, out, w);
			if (res != w) {
				if (res < 0)
					perror("write");
				else
					write(2, "short write\n", 12);
				exit(4);
			}
		} else { // size == 0
			r = read(0, in, 65536);
			if (r < 0) {
				perror("read"); 
				exit(2);
			}
			res = write(1, in, r);
			if (res != r) {
				if (res < 0)
					perror("write");
				else
					write(2, "short write\n", 12);
				exit(4);
			}
		}
	}
}
